
#ifndef AY38900_H
#define AY38900_H

#include <windows.h>

#include "Processor.h"
#include "Peripheral.h"
#include "CP1610.h"
#include "SignalLine.h"
#include "MemoryAlias.h"
#include "BackTabRAM.h"
#include "ROM.h"
#include "GRAM.h"
#include "MOB.h"
#include "AY38900_Registers.h"
#include "PlugIn.h"
#include "VideoOutputDevice.h"
#include "InputDevice.h"
#include "ClockDevice.h"

class AY38900 : public Processor, public Peripheral
{

    friend class AY38900_Registers;

    public:
        AY38900();
        void init(CP1610* cpu, SignalLine* SR1, SignalLine* SR2, SignalLine* SST);
        INT32 getProcessorCount();
        void getProcessor(INT32 i, Processor** p);
        INT32 getMemoryCount();
        void getMemory(INT32 i, Memory** m);
        INT32 getClockSpeed();
        void reset();
        INT32 tick();
        void setGROMImage(INT32* gromImage);
        BOOL hasGROMImage();
        void changePlugIn(PlugIn* plugIn);
        void setMessage(CHAR* message);
        void setMessage(CHAR* message, INT32 numSeconds);

        static const INT32 TICK_LENGTH_SCANLINE;
        static const INT32 TICK_LENGTH_FRAME;
        static const INT32 TICK_LENGTH_VBLANK;
        static const INT32 TICK_LENGTH_START_ACTIVE_DISPLAY;
        static const INT32 TICK_LENGTH_IDLE_ACTIVE_DISPLAY;
        static const INT32 TICK_LENGTH_FETCH_ROW;
        static const INT32 TICK_LENGTH_RENDER_ROW;
        static const INT32 LOCATION_BACKTAB;
        static const INT32 LOCATION_GROM;
        static const INT32 LOCATION_GRAM;
        static const INT32 LOCATION_COLORSTACK;
        static const INT32 FOREGROUND_BIT;

    private:
        void setGraphicsBusVisible(BOOL visible);
        void renderFrame();
        void displayFrame();
        void setFrameSkip(INT32 frameSkip);
        INT32 getFrameSkip();
        BOOL somethingChanged();
        void markClean();
        void renderBorders();
        void renderMOBs();
        void renderBackground();
        void copyBackgroundBufferToStagingArea();
        void renderForegroundBackgroundMode();
        void renderColorStackMode();
        void copyMOBsToStagingArea();
        void renderLine(INT32 nextByte, INT32 x, INT32 y,
                UINT8 fgcolor, UINT8 bgcolor);
        void renderColoredSquares(INT32 x, INT32 y, UINT8 color0, UINT8 color1,
                UINT8 color2, UINT8 color3);
        void renderMessage();
        void renderCharacter(char c, INT32 x, INT32 y);
        void determineMOBCollisions();
        BOOL mobsCollide(INT32 mobNum0, INT32 mobNum1);

        PlugIn*           plugIn;
        VideoOutputDevice* vod;
        InputDevice*      id;
        ClockDevice*      clock;
        CP1610*           cpu;
        SignalLine*       SR1;
        SignalLine*       SR2;
        SignalLine*       SST;
        MemoryAlias       memorySegments[10];
        BackTabRAM        backtab;
        ROM               grom;
        GRAM              gram;
        UINT8             stagingBankData[320*192];
        UINT8             backgroundDataBuffer[192][320];
        AY38900_Registers registers;
        MOB            mobs[8];

        INT64             startTime;
        INT64             missedFrameBuffer;
        INT32             frameCount;
        INT32             frameSkipCounter;
        INT32             frameSkip;
        CHAR*             message;
        BOOL              messageChanged;
        INT32             messageCounter;

        //state info
        INT32           mode;
        BOOL            previousDisplayEnabled;
        BOOL            displayEnabled;
        BOOL            colorStackMode;
        BOOL            colorModeChanged;
        BOOL            bordersChanged;
        BOOL            colorStackChanged;
        BOOL            offsetsChanged;
        BOOL            hasGROM;

        //register info
        UINT8   borderColor;
        BOOL    blockLeft;
        BOOL    blockTop;
        INT32   horizontalOffset;
        INT32   verticalOffset;

        //mob info
        BOOL     mobBuffers[8][16][128];

/*
        void save(SaveOutputStream os) throws IOException {
        void load(LoadInputStream is) throws IOException {
*/

};

#endif
