
#include "HandController.h"

const INT32 HandController::ID_KEYPAD_ONE   = 0;
const INT32 HandController::ID_KEYPAD_TWO   = 1;
const INT32 HandController::ID_KEYPAD_THREE = 2;
const INT32 HandController::ID_KEYPAD_FOUR  = 3;
const INT32 HandController::ID_KEYPAD_FIVE  = 4;
const INT32 HandController::ID_KEYPAD_SIX   = 5;
const INT32 HandController::ID_KEYPAD_SEVEN = 6;
const INT32 HandController::ID_KEYPAD_EIGHT = 7;
const INT32 HandController::ID_KEYPAD_NINE  = 8;
const INT32 HandController::ID_KEYPAD_CLEAR = 9;
const INT32 HandController::ID_KEYPAD_ZERO  = 10;
const INT32 HandController::ID_KEYPAD_ENTER = 11;
const INT32 HandController::ID_ACTION_BUTTON_TOP          = 12;
const INT32 HandController::ID_ACTION_BUTTON_BOTTOM_LEFT  = 13;
const INT32 HandController::ID_ACTION_BUTTON_BOTTOM_RIGHT = 14;
const INT32 HandController::ID_DISC_NORTH            = 15;
const INT32 HandController::ID_DISC_NORTH_EAST       = 16;
const INT32 HandController::ID_DISC_EAST             = 17;
const INT32 HandController::ID_DISC_SOUTH_EAST       = 18;
const INT32 HandController::ID_DISC_SOUTH            = 19;
const INT32 HandController::ID_DISC_SOUTH_WEST       = 20;
const INT32 HandController::ID_DISC_WEST             = 21;
const INT32 HandController::ID_DISC_NORTH_WEST       = 22;

const INT32 HandController::BUTTON_OUTPUT_VALUES[15] = {
    0x81, //OUTPUT_KEYPAD_ONE
    0x41, //OUTPUT_KEYPAD_TWO
    0x21, //OUTPUT_KEYPAD_THREE
    0x82, //OUTPUT_KEYPAD_FOUR
    0x42, //OUTPUT_KEYPAD_FIVE
    0x22, //OUTPUT_KEYPAD_SIX
    0x84, //OUTPUT_KEYPAD_SEVEN
    0x44, //OUTPUT_KEYPAD_EIGHT
    0x24, //OUTPUT_KEYPAD_NINE
    0x88, //OUTPUT_KEYPAD_CLEAR
    0x48, //OUTPUT_KEYPAD_ZERO
    0x28, //OUTPUT_KEYPAD_ENTER
    0xA0, //OUTPUT_ACTION_BUTTON_TOP
    0x60, //OUTPUT_ACTION_BUTTON_BOTTOM_LEFT
    0xC0  //OUTPUT_ACTION_BUTTON_BOTTOM_RIGHT
};

const INT32 HandController::DIRECTION_OUTPUT_VALUES[16] = {
    0x04, //OUTPUT_DISC_NORTH
    0x14, //OUTPUT_DISC_NORTH_NORTH_EAST
    0x16, //OUTPUT_DISC_NORTH_EAST
    0x06, //OUTPUT_DISC_EAST_NORTH_EAST
    0x02, //OUTPUT_DISC_EAST
    0x12, //OUTPUT_DISC_EAST_SOUTH_EAST
    0x13, //OUTPUT_DISC_SOUTH_EAST
    0x03, //OUTPUT_DISC_SOUTH_SOUTH_EAST
    0x01, //OUTPUT_DISC_SOUTH
    0x11, //OUTPUT_DISC_SOUTH_SOUTH_WEST
    0x19, //OUTPUT_DISC_SOUTH_WEST
    0x09, //OUTPUT_DISC_WEST_SOUTH_WEST
    0x08, //OUTPUT_DISC_WEST
    0x18, //OUTPUT_DISC_WEST_NORTH_WEST
    0x1C, //OUTPUT_DISC_NORTH_WEST
    0x0C  //OUTPUT_DISC_NORTH_NORTH_WEST
};

const double PI = 3.14159265358979323846;

const float HandController::vectorParse = sin(PI/4.0);

HandController::HandController() {}

HandController::HandController(INT32 controlID)
{
    init(controlID);
}

void HandController::init(INT32 controlID) {
    this->controlID = controlID;
}

void HandController::reset() {
    inputValue = 0xFF;
}

INT32 HandController::getControlCount() {
    return 23;
}

void HandController::getControlDescription(INT32 controlNum, CHAR* desc) {
    //nothing for now
}

void HandController::setControlInputSignal(INT32 controlNum,
        InputSignal* inputSignal)
{
    inputSignals[controlNum] = inputSignal;
}

void HandController::getControlInputSignal(INT32 controlNum,
        InputSignal** signal)
{
    (*signal) = inputSignals[controlNum];
}

void HandController::setInputDevice(InputDevice* inputDevice) {
    this->inputDevice = inputDevice;
    if (inputDevice == NULL)
        return;

    InputSignal* defaultSignal;
    for (INT32 i = 0; i < 23; i++) {
        inputDevice->getDefaultConfiguration(controlID, i, &defaultSignal);
        this->inputSignals[i] = defaultSignal;
    }
}

void HandController::setOutputValue(INT32 value) {}

INT32 HandController::getInputValue() {
    return inputValue;
}

void HandController::poll() {
    inputValue = 0;

    for (INT32 i = 0; i < 15; i++) {
        if (inputDevice->getSignalValue(inputSignals[i]) ==
                InputDevice.SIGNAL_ON)
            inputValue |= BUTTON_OUTPUT_VALUES[i];
    }

    //evaluate the disc
    float neswVector =
            (inputDevice->getSignalValue(inputSignals[ID_DISC_NORTH_EAST]) -
            inputDevice->getSignalValue(inputSignals[ID_DISC_SOUTH_WEST])) *
            vectorParse;
    float nwseVector =
            (inputDevice->getSignalValue(inputSignals[ID_DISC_NORTH_WEST]) -
            inputDevice->getSignalValue(inputSignals[ID_DISC_SOUTH_EAST])) *
            vectorParse;
    float yPos = inputDevice->getSignalValue(inputSignals[ID_DISC_NORTH]) -
            inputDevice->getSignalValue(inputSignals[ID_DISC_SOUTH]) +
            nwseVector + neswVector;
    float xPos = inputDevice->getSignalValue(inputSignals[ID_DISC_EAST]) -
            inputDevice->getSignalValue(inputSignals[ID_DISC_WEST]) -
            nwseVector + neswVector;

    if (xPos != 0 || yPos != 0) {
        double positionInRadians = (atan2(-xPos, -yPos)+PI);
        double offset = (2.0 * PI)/16.0;
        INT32 directionValue = DIRECTION_OUTPUT_VALUES[(INT32)(
                (positionInRadians+(offset/2.0))/offset) & 0x0F];
        inputValue |= directionValue;
    }

    inputValue = (0xFF ^ inputValue);
}

/*
private final static String[] descriptionKeys = {
        "InputKeyPadOne",
        "InputKeyPadTwo",
        "InputKeyPadThree",
        "InputKeyPadFour",
        "InputKeyPadFive",
        "InputKeyPadSix",
        "InputKeyPadSeven",
        "InputKeyPadEight",
        "InputKeyPadNine",
        "InputKeyPadClear",
        "InputKeyPadZero",
        "InputKeyPadEnter",
        "InputActionButtonsTop",
        "InputActionButtonBottomLeft",
        "InputActionButtonBottomRight",
        "InputDiscNorth",
        "InputDiscNorthEast",
        "InputDiscEast",
        "InputDiscSouthEast",
        "InputDiscSouth",
        "InputDiscSouthWest",
        "InputDiscWest",
        "InputDiscNorthWest"
    };
*/


