
#ifndef HANDCONTROLLER_H
#define HANDCONTROLLER_H

#include <math.h>
#include "IODevice.h"
#include "InputDevice.h"
#include "InputSignal.h"

class EmulationDirector;

class HandController : public IODevice
{

    friend class EmulationDirector;

    public:
        HandController();
        HandController(INT32 controlID);
        void init(INT32 controlID);
        void reset();
        INT32 getControlCount();
        void getControlDescription(INT32 controlNum, CHAR* desc);
        void setControlInputSignal(INT32 controlNum, InputSignal* inputSignal);
        void getControlInputSignal(INT32 controlNum, InputSignal** signal);
        void setInputDevice(InputDevice* inputDevice);
        void setOutputValue(INT32 value);
        INT32 getInputValue();
        void poll();

    private:
        INT32            controlID;
        InputDevice*     inputDevice;
        INT32            inputValue;
        InputSignal*     inputSignals[23];

        static const INT32 BUTTON_OUTPUT_VALUES[15];
        static const INT32 DIRECTION_OUTPUT_VALUES[16];
        static const INT32 ID_KEYPAD_ONE;
        static const INT32 ID_KEYPAD_TWO;
        static const INT32 ID_KEYPAD_THREE;
        static const INT32 ID_KEYPAD_FOUR;
        static const INT32 ID_KEYPAD_FIVE;
        static const INT32 ID_KEYPAD_SIX;
        static const INT32 ID_KEYPAD_SEVEN;
        static const INT32 ID_KEYPAD_EIGHT;
        static const INT32 ID_KEYPAD_NINE;
        static const INT32 ID_KEYPAD_CLEAR;
        static const INT32 ID_KEYPAD_ZERO;
        static const INT32 ID_KEYPAD_ENTER;
        static const INT32 ID_ACTION_BUTTON_TOP;
        static const INT32 ID_ACTION_BUTTON_BOTTOM_LEFT;
        static const INT32 ID_ACTION_BUTTON_BOTTOM_RIGHT;
        static const INT32 ID_DISC_NORTH;
        static const INT32 ID_DISC_NORTH_EAST;
        static const INT32 ID_DISC_EAST;
        static const INT32 ID_DISC_SOUTH_EAST;
        static const INT32 ID_DISC_SOUTH;
        static const INT32 ID_DISC_SOUTH_WEST;
        static const INT32 ID_DISC_WEST;
        static const INT32 ID_DISC_NORTH_WEST;
        static const float vectorParse;

/*
    private final static String[] descriptionKeys = {
            "InputKeyPadOne",
            "InputKeyPadTwo",
            "InputKeyPadThree",
            "InputKeyPadFour",
            "InputKeyPadFive",
            "InputKeyPadSix",
            "InputKeyPadSeven",
            "InputKeyPadEight",
            "InputKeyPadNine",
            "InputKeyPadClear",
            "InputKeyPadZero",
            "InputKeyPadEnter",
            "InputActionButtonsTop",
            "InputActionButtonBottomLeft",
            "InputActionButtonBottomRight",
            "InputDiscNorth",
            "InputDiscNorthEast",
            "InputDiscEast",
            "InputDiscSouthEast",
            "InputDiscSouth",
            "InputDiscSouthWest",
            "InputDiscWest",
            "InputDiscNorthWest"
        };
*/

};

#endif
