
#include <iostream>
#include <stdio.h>
#include "ProcessorBus.h"

using namespace std;

ProcessorBus::ProcessorBus()
    : processorCount(0)
{
}

void ProcessorBus::reset()
{
    paused = FALSE;
    INT32 i;

    UINT64 totalClockSpeed = (UINT64)processors[0]->getClockSpeed();
    for (i = 1; i < processorCount; i++) {
        totalClockSpeed = lcm(totalClockSpeed,
                ((UINT64)processors[i]->getClockSpeed()));
    }

    for (i = 0; i < processorCount; i++) {
        processors[i]->reset();
        processorsIdle[i] = FALSE;
        processorTicks[i] = 0;
        processorTickFactors[i] = (totalClockSpeed /
                ((UINT64)processors[i]->getClockSpeed()));
    }

    currentProcessor = 0;
    tickDelta = 0;
    clock = 1;
    deltaPass = TRUE;
}

void ProcessorBus::addProcessor(Processor* p)
{
    p->changePlugIn(plugIn);
    processors[processorCount] = p;
    processorCount++;
}

void ProcessorBus::removeProcessor(Processor* p)
{
    for (INT32 i = 0; i < processorCount; i++) {
        if (processors[i] == p) {
            for (INT32 j = i; j < (processorCount-1); j++)
                processors[j] = processors[j+1];
            processorCount--;
        }
    }
}

void ProcessorBus::run()
{
    running = TRUE;
    while (running) {
        //skip this processor if it has been idled
        if (!processorsIdle[currentProcessor]) {
            processorTicks[currentProcessor] -= tickDelta;
 
            //if the clock just caught up to the processor, and
            //if the processor is not about to go idle, then run it
            if (processorTicks[currentProcessor] == 0) {
                if (processors[currentProcessor]->isIdle)
                    processorsIdle[currentProcessor] = TRUE;
                else
                    processorTicks[currentProcessor] =
                            (((UINT64)processors[currentProcessor]->tick()) *
                                processorTickFactors[currentProcessor]);
            }
        }
 
        //move to the next processor
        currentProcessor++;
 
        //if we've now run through all of the processors, go back to
        //the beginning, awaken any processors that have come out of
        //an idle state, and determine the next tick delta
        if (currentProcessor == processorCount) {
            currentProcessor = 0;
 
            //determine the next tick delta
            tickDelta = 0xFFFFFFFFFFFFFFFF;
            for (int i = 0; i < processorCount; i++) {
                //wake up any processors that want to wake up
                processorsIdle[i] = (processorsIdle[i] &&
                        processors[i]->isIdle);
 
                //if the processor is not idle by this point, use it
                //to calculate the tick delta
                if (!processorsIdle[i] && processorTicks[i] < tickDelta)
                    tickDelta = processorTicks[i];
            }
 
            //move the clock forward by the tick delta amount
            clock += tickDelta;
        }
 
        while (paused && running)
            processors[0]->tick();
    }
}

void ProcessorBus::stop() {
    this->running = FALSE;
}

void ProcessorBus::setPaused(BOOL paused) {
    this->paused = paused;
}

BOOL ProcessorBus::isPaused() {
    return paused;
}

void ProcessorBus::changePlugIn(PlugIn* plugIn) {
    if (this->plugIn == plugIn)
        return;

    this->plugIn = plugIn;
    for (INT32 i = 0; i < processorCount; i++)
        processors[i]->changePlugIn(plugIn);
}

UINT64 lcm(UINT64 a, UINT64 b) {
    /**
     * This is an implementation of Euclid's Algorithm for determining
     * the greatest common denominator of two numbers.
     */
    UINT64 m = a;
    UINT64 n = b;
    UINT64 r = m % n;
    while (r != 0) {
        m = n;
        n = r;
        r = m % n;
    }
    UINT64 gcd = n;
    return (a/gcd)*b;
}

UINT64 gcd(UINT64 m, UINT64 n) {
    return n;
}

