
#ifndef DIRECTXVIDEOOUTPUTDEVICE_H
#define DIRECTXVIDEOOUTPUTDEVICE_H

#include <D3d8.h>
#include <D3dx8tex.h>
#include <D3d8types.h>
#include <windows.h>
#include "Intellivision.h"
#include "VideoOutputDevice.h"
#include "DirectInputDevice.h"
#include "DirectXAudioOutputDevice.h"

class DirectXVideoOutputDevice : public VideoOutputDevice
{

    friend class DirectInputDevice;
    friend class DirectXAudioOutputDevice;
    friend DWORD WINAPI CreateDirectXWindow(LPVOID param);
    friend LRESULT CALLBACK MsgProc(HWND hWnd, UINT msg, WPARAM wParam,
            LPARAM lParam);

    public:
        DirectXVideoOutputDevice();
        void setFullScreenMode(BOOL b);
        INT32 init(Intellivision* inty);
        void displayImage(UINT8* image);
        void release(Intellivision* inty);

    private:
        INT32 initWindow();
        INT32 initDirectX();
        INT32 createResources();
        void destroyResources();
        void destroyDirectX();
        void getDisplayModes();
        INT32 copyAndDisplayImage();

        D3DDISPLAYMODE* modes;
        INT32           modeCount;
        D3DDISPLAYMODE* defaultFullScreenMode;

        D3DPRESENT_PARAMETERS presentation;

        //reference to the current Intellivision
        Intellivision* inty;

        //reference to the current image data
        UINT8* imageData;

        //display parameters
        BOOL      fullScreen;
        BOOL      filtered;

        //display device data
        HWND                    m_hWnd;
        DWORD                   m_directXThreadID;
        HANDLE                  m_hDirectXThreadHandle;
        HANDLE                  m_hEventObject;
        IDirect3D8*             m_pD3D;
        IDirect3DDevice8*       m_pd3dDevice;
        IDirect3DVertexBuffer8* m_pVB;
        IDirect3DTexture8*      m_pMemTexture;
        WORD*                   m_pMemPalette;
        D3DSURFACE_DESC         m_pMemDesc;
        IDirect3DTexture8*      m_pCardTexture;

        static const INT32 DEFAULT_WINDOW_WIDTH;
        static const INT32 DEFAULT_WINDOW_HEIGHT;

};

#endif
